@@ -26,9 +26,16 @@ test_that("Hanging commas don't break things", {
   expect_equal(as.character(tagList("hi",)), "hi")
   expect_equal(as.character(div("one",)), "<div>one</div>")
   # Multiple commas still throw
-  expect_error(as.character(div("one",,)), "is empty")
+  err_comma_multiple <- expect_error(as.character(div("one",,)))
   # Non-trailing commas still throw
-  expect_error(as.character(div(,"one",)), "is empty")
+  err_comma_leading <- expect_error(as.character(div(,"one",)))
+
+  # rlang > 1.0.6 changed the error message, so only run
+  # snapshot testing of the error message with the new version
+  skip_if_not_installed("rlang", "1.0.6.9000")
+  local_edition(3)
+  expect_snapshot(err_comma_multiple)
+  expect_snapshot(err_comma_leading)
 })
 
 
@@ -828,18 +835,58 @@ test_that("Indenting can be controlled/suppressed", {
 test_that("cssList tests", {
   expect_identical(NULL, css())
   expect_identical(NULL, css())
+
+  # Regular CSS properties with conveniences
+  expect_identical(css(font.size = "12px"), "font-size:12px;")
+  expect_identical(css(font_size = "12px"), "font-size:12px;")
+  expect_identical(css(fontSize = "12px"), "font-size:12px;")
+  expect_identical(css(`font-style` = "italic"), "font-style:italic;")
+  expect_null(css(font.variant = NULL))
+  expect_identical(
+    css(.webkit.animation = "fade-in 1s"),
+    "-webkit-animation:fade-in 1s;"
+  )
+  expect_identical(
+    css(font.family = 'Helvetica, "Segoe UI"'),
+    "font-family:Helvetica, \"Segoe UI\";"
+  )
+  expect_identical(
+    css("font-weight!" = factor("bold")),
+    "font-weight:bold !important;"
+  )
+  expect_identical(
+    css(padding = c("10px", "9px", "8px")),
+    "padding:10px 9px 8px;"
+  )
+
+  # CSS variables
+  expect_identical(css("--_foo" = "bar"), "--_foo:bar;")
+  expect_identical(css("--fooBar" = "baz"), "--fooBar:baz;")
+  expect_identical(css("--foo_bar" = "baz"), "--foo_bar:baz;")
+  expect_identical(css("--_foo!" = "bar"), "--_foo:bar !important;")
+  expect_identical(css("--fooBar!" = "baz"), "--fooBar:baz !important;")
+  expect_identical(css("--foo_bar!" = "baz"), "--foo_bar:baz !important;")
+
+  # Mix of CSS variables and regular CSS properties
   expect_identical(
     css(
-      font.family = 'Helvetica, "Segoe UI"',
-      font_size = "12px",
-      `font-style` = "italic",
-      font.variant = NULL,
-      "font-weight!" = factor("bold"),
-      padding = c("10px", "9px", "8px")
+      "--empty" = NULL,
+      "--_foo" = "bar",
+      `_foo` = "bar",
+      "--foo_bar" = "baz",
+      foo_bar = "baz",
+      "--fooBar" = "baz",
+      fooBar = "baz",
     ),
-    "font-family:Helvetica, \"Segoe UI\";font-size:12px;font-style:italic;font-weight:bold !important;padding:10px 9px 8px;"
+    "--_foo:bar;-foo:bar;--foo_bar:baz;foo-bar:baz;--fooBar:baz;foo-bar:baz;"
   )
 
+  # Lists can be spliced
+  expect_identical(css(!!!list(a = 1, b = 2)), "a:1;b:2;")
+
+  # Factors are coerced to strings
+  expect_identical(css(a = factor('a')), "a:a;")
+
   # Unnamed args not allowed
   expect_error(css("10"))
   expect_error(css(1, b=2))
@@ -1153,3 +1200,29 @@ test_that("flattenTagAttribs", {
   # b values are collected
   expect_equal(flatAttribs, list(a = "2", b = c("1", "3")))
 })
+
+test_that("htmlDependency() can be included in rmarkdown via knit_print", {
+  skip_if_not_installed("knitr")
+
+  dep <- htmlDependency(
+    "dummytestdep",
+    "1.0",
+    c(href = "http://example.com/"),
+    script = "test.js"
+  )
+
+  dep_knitr <- knit_print.html_dependency(dep)
+
+  expect_s3_class(dep_knitr, "knit_asis")
+  expect_equal(attr(dep_knitr, "knit_meta")[[1]], dep)
+})
+
+test_that("includeHTML() warns if full document is detected", {
+  tmp_html <- withr::local_tempfile(fileext = ".html")
+
+  writeLines("<html><body><p>test</p></body></html>", tmp_html)
+  expect_warning(includeHTML(tmp_html))
+
+  save_html(p("test"), tmp_html)
+  expect_warning(includeHTML(tmp_html))
+})
